reference_structures_to_render = ["TH", "SCs", "SCm", "PAG"]

metrics_and_axis_labels = {
    "n_cells_in_region": "Cells ( Number )",
    "percentage": "Cells ( % )",
    "cells_per_mm3": "Cell density ( cells / mm3 )",
    "percent_of_reference_region": "Cells ( % of ref )",
}

camera = {
    "pos": (-15854, -61680, 23155),
    "viewup": (1, 0, -1),
    "clippingRange": (57281, 96305),
}

zoom = 1.5

additional_obj_color = "k"
